#=======================================================================
#	$Id: Multi.pm,v 1.3 2005/11/25 12:57:24 pythontech Exp $
#	Treat multiple backends as a single entity.
#	Copyright (C) 2004,2005  Python Technology Limited
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
#	02111-1307, USA.
#-----------------------------------------------------------------------
#	Updates go in the first one.
#=======================================================================
package PythonTech::Triples::Multi;
use strict;
use PythonTech::Triples;

our @ISA = qw(PythonTech::Triples);

#-----------------------------------------------------------------------
#	Create a multiple triplestore combining others
#	  dbs		List of triplestores
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $props = ref($_[0]) eq 'HASH' ? {%{$_[0]}} : {@_};
    my @dbs = @{$props->{'dbs'}};
    my $self = {
	dbs => [@dbs],
	main => $dbs[0],
    };
    bless $self, $class;
    return $self;
}

sub configure {
    my($self, $props) = shift;
    while (my($name,$val) = each %$props) {
	if ($name eq 'dbs') {
	    push @{$self->{'dbs'}}, @$val;
	} else {
	    die "$self: Unknown configuration option `$name'\n";
	}
    }
}

#-----------------------------------------------------------------------
#	Add a statement in the (first) database
#-----------------------------------------------------------------------
sub add {
    my($self, $subj,$pred,$obj) = @_;
    $self->{'main'}->add($subj,$pred,$obj);
}

#-----------------------------------------------------------------------
#	Remove a statement from the (first) database
#-----------------------------------------------------------------------
sub remove {
    my($self, $subj,$pred,$obj) = @_;
    $self->{'main'}->remove($subj,$pred,$obj);
}

#-----------------------------------------------------------------------
#	Generic query.
#	May get the same statement returned multiple times, if it is
#	present in more then one of the underlying stores.
#-----------------------------------------------------------------------
sub iterate {
    my($self, $subj,$pred,$obj,$cb) = @_;
    foreach my $db (@{$self->{'dbs'}}) {
	my $res = $db->iterate($subj,$pred,$obj,$cb);
	return $res if $res;
    }
    return;
}

#-----------------------------------------------------------------------
#	Commit any pending changes
#-----------------------------------------------------------------------
sub commit {
    my($self) = @_;
    $self->{'main'}->commit;
}

#-----------------------------------------------------------------------
#	Cancel any pending changes
#-----------------------------------------------------------------------
sub rollback {
    my($self) = @_;
    $self->{'main'}->rollback;
}

#-----------------------------------------------------------------------
#	Disconnect from / reconnect to backend
#-----------------------------------------------------------------------
sub disconnect {
    my($self) = @_;
    $self->{'main'}->disconnect;
}

sub reconnect {
    my($self, @args) = @_;
    $self->{'main'}->reconnect(@args);
}

1;
